/* Netmask Calculator v0.8 - calculate size of net masks
 *
 * Copyright (C) 2004 Giuseppe Caulo <giusc@softzone.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>
#ifndef _WIN32
# include <netinet/in.h>
#else
# include <winsock.h>
/* # if defined(_MSVC_VER) */
#  define snprintf _snprintf
#  pragma comment (lib, "wsock32.lib")
/* # endif */
#endif

/* typedef unsigned char u_char; */
typedef unsigned long ip_t;

ip_t my_inet_addr (char *ip)
{
  ip_t ret = 0;
  int bit;
  int a,b,c,d;
  if (sscanf(ip, "%d.%d.%d.%d",&a,&b,&c,&d) != 4)
    return -1;
  if ((a < 0 || a > 255) || (b < 0 || b > 255) ||
      (c < 0 || c > 255) || (d < 0 || d > 255))
    return -1;
  for (bit = 0; bit < 8; ++bit)
  {
    if (a & (1 << bit)) ret |= (1 << bit);
    if (b & (1 << bit)) ret |= (1 << (bit + 8));
    if (c & (1 << bit)) ret |= (1 << (bit + 16));
    if (d & (1 << bit)) ret |= (1 << (bit + 24));
  }

  return (ret);
}

unsigned long my_atoul (char * s)
{
    unsigned long ret = 0;
    while ((*s >= '0') && (*s <= '9'))
    {
        ret *= 10;
        ret += ((*s) - '0');
        s++;
    }
    return ret;
}


int main(int argc, char *argv[]) {
  int subnet=-1;
  char *range = NULL;
  ip_t user_ip, start_ip, end_ip, nm_ip;
  ip_t i;
  unsigned char show_help = 1;
  int arg, verbose = 0;


   /* Scan command Line for parameters */
  for (arg=1; arg<argc; arg++) {
	  if (*argv[arg] == '-') {
	    /* non-interactive */
	    if (*(argv[arg] + 1) == 'v') {
			if (!verbose) {
			  while (argv[arg][verbose+1] && verbose<2)
			    verbose++;
			} else {
			  printf("You can't specify verbosity more than once\n");
			  return -1;
			}
  	    /* help */
	    } else if (!strcmp(argv[arg], "-h") || !strcmp(argv[arg], "-?")) {
			break;
		} else {
			printf("Unknown option: '%s'\n", argv[arg]);
			return -1;
		}
	  } else {
		range = strchr(argv[arg], '/');
		if (range == NULL) {
		  if (verbose > 0)
		    printf("no subnet specified, assuming /32\n");
		  subnet = 32;
		} else {
		  *(range++) = 0;
		}
		/* Parse the ip parts */
		if ((user_ip = my_inet_addr(argv[arg])) == -1) {
		  printf("Invalid ip: %s (Should be in dot decimal format)\n", argv[arg]);
		  return -1;
		}
		//user_ip = ntohl(inet_addr(argv[arg]));
		user_ip = ntohl(user_ip);
		/* Parse the range parts */
		if (range != NULL && strchr(range, '.')) {
		  if ((nm_ip = my_inet_addr(range)) == -1) {
		    printf("Invalid netmask specified. Giving up "
			  "(should be in the form 255.255.255.0)\n");
		    return -1;
		  }
		  nm_ip = ntohl(nm_ip);
		  if (!nm_ip) {
		    /* process almost the internet */
		    start_ip = ntohl(my_inet_addr("1.0.0.0"));
		    end_ip = ntohl(my_inet_addr("255.255.255.254"));
		  } else {
		    start_ip = user_ip & nm_ip;
		    end_ip = start_ip + (ntohl(my_inet_addr("255.255.255,255")) - nm_ip);
		  }
		} else {
		  if (subnet == -1)
		    subnet = my_atoul(range);
		  if (subnet > 32 && subnet < 0) {
		    printf("blah, invalid subnet, should be from 0 to 32, assuming 32\n");
		    subnet = 32;
		  }
		  start_ip = user_ip & (unsigned long) (0 - (1<<(32 - subnet)));
		  end_ip = user_ip | (unsigned long)  ((1<<(32 - subnet)) - 1);
		}
		show_help = 0;
	  }
  }

  if (show_help) {
	printf ("Netmask Calculator v0.8\nWritten by Giuseppe Caulo.\n\n"
	        "Usage: %s [-h] [-v[v]] ip/[subnet|netmask]\n", argv[0]);
    return -1;
  }

  if (verbose > 0) {
    printf("List of selected address:\n");
    if (verbose > 1)
      printf("start_ip = %lu\nend_ip = %lu\n", start_ip, end_ip);
  }
  for (i = start_ip; i<= end_ip; i++) {
    if (i < (1 << 24)) {
	  if (verbose > 0)
	    fprintf(stderr, "ERROR!!! %lu is an impossible ip addy\n", i);
	  return -2;
    }
    if (verbose > 1) {
	  printf("%lu: ", i);
	}
    printf("%lu.%lu.%lu.%lu\n", (i >> 24) & 0xff,
			 (i >> 16) & 0xff, (i >> 8) & 0xff, i & 0xff);
  }
  return 0;
}
